a__all__ = ['HmiHandler']

import threading
import serial
import time
from .data_to_byte import *
from .decoder import *
from .util import *

class HmiHandler_serial(threading.Thread):
    pass

    def putArray(self, data):
        pass

    def putStruct(self, data):
        pass

    def putString(self, data):
        pass

    def putSync(self):
        pass

    def get(self):
        pass

    def getArray(self):
        pass

    def getStruct(self):
        pass

    def getString(self):
        pass

    def getAvailableDataNum(self):
        pass



__all__ = ['HmiHandler']

import threading
import serial
import time
from .data_to_byte import *
from .decoder import *
from .util import *

class HmiHandler(threading.Thread):
    _ser = serial.Serial()
    _dataBuffer = list()
    _timeout = int()

    def __init__(
        self,
        port=None,
        baudrate=38400,
        timeout=None
    ):
        super(HmiHandler, self).__init__(name = 'hmi')
        self._ser.port = port
        self._ser.baudrate = baudrate
        self._ser.timeout = 0.01
        self._timeout = timeout
        self.lock = threading.Lock()

    def setSerial(
        port,
        baudrate=38400,
    ):
        self._ser.port = port
        self._ser.baudrate = baudrate

    def run(self):
        de = Decoder()
        self._dataBuffer = list()
        self._ser.open()

        while (self._ser.isOpen()):
            try:
                self.lock.acquire()
                ch = self._ser.read(1)
                self.lock.release()
            except serial.serialutil.SerialException as e:
                raise AsaHmiException()
            else:
                if ch != b'':
                    self.lock.acquire()
                    de.add_text(ch)
                    type, data = de.get()
                    if type is 0:
                        pass
                    else:
                        self._dataBuffer.append((type, data))
                    self.lock.release()

    def putArray(self, data):
        b = encodeArToPac(data)
        self._ser.write(b)

    def putStruct(self, data):
        b = encodeStToPac(data)
        self._ser.write(b)

    def putString(self, data):
        b = data.encode('utf-8')
        self._ser.write(b)

    def putSync(self):
        self._ser.write(b'sync\n')

    def get(self):
        if not self._ser.is_open:
            raise SerialNotOpenException
        if self._timeout is not None and self._timeout != 0:
            timeout = time.time() + self._timeout
        else:
            timeout = None
        data = None
        while self._ser.is_open:
            if len(self._dataBuffer) >= 1:
                data = self._dataBuffer.pop(0)
                break
            # check for timeout now, after data has been read.
            # useful for timeout = 0 (non blocking) read
            if timeout and time.time() > timeout:
                break
        return data

    def getArray(self):
        if not self._ser.is_open:
            raise SerialNotOpenException
        if self._timeout is not None and self._timeout != 0:
            timeout = time.time() + self._timeout
        else:
            timeout = None
        data = None
        while self._ser.is_open:
            if len(self._dataBuffer) >= 1:
                if self._dataBuffer[0][0] == 1:
                    data = self._dataBuffer.pop(0)
                    break
                else:
                    break
            # check for timeout now, after data has been read.
            # useful for timeout = 0 (non blocking) read
            if timeout and time.time() > timeout:
                break
        return data

    def getStruct(self):
        if not self._ser.is_open:
            raise SerialNotOpenException
        if self._timeout is not None and self._timeout != 0:
            timeout = time.time() + self._timeout
        else:
            timeout = None
        data = None
        while self._ser.is_open:
            if len(self._dataBuffer) >= 1:
                if self._dataBuffer[0][0] == 2:
                    data = self._dataBuffer.pop(0)
                    break
                else:
                    break
            # check for timeout now, after data has been read.
            # useful for timeout = 0 (non blocking) read
            if timeout and time.time() > timeout:
                break
        return data

    def getString(self):
        if not self._ser.is_open:
            raise SerialNotOpenException
        if self._timeout is not None and self._timeout != 0:
            timeout = time.time() + self._timeout
        else:
            timeout = None
        data = None
        while self._ser.is_open:
            if len(self._dataBuffer) >= 1:
                if len(self._dataBuffer) >= 1:
                    if self._dataBuffer[0][0] == 3:
                        data = self._dataBuffer.pop(0)
                        break
                    else:
                        break
            # check for timeout now, after data has been read.
            # useful for timeout = 0 (non blocking) read
            if timeout and time.time() > timeout:
                break
        return data

    def getAvailableDataNum(self):
        return len(self._dataBuffer)



from struct import *
import numpy as np

def getNpType(typeNum):
    if typeNum == 0: # int8_t
        return np.int8
    elif typeNum == 1: # int16_t
        return np.int16
    elif typeNum == 2: # int32_t
        return np.int32
    elif typeNum == 3: # int64_t
        return np.int64
    elif typeNum == 4: # uint8_t
        return np.uint8
    elif typeNum == 5: # uint16_t
        return np.uint16
    elif typeNum == 6: # uint32_t
        return np.uint32
    elif typeNum == 7: # uint64_t
        return np.uint64
    elif typeNum == 8: # f32
        return np.float32
    elif typeNum == 9: # f64
        return np.float64
    else:
        return False

def getTypeStr(typeNum):
    if typeNum == 0:
        return 'i8'
    elif typeNum == 1:
        return 'i16'
    elif typeNum == 2:
        return 'i32'
    elif typeNum == 3:
        return 'i64'
    elif typeNum == 4:
        return 'ui8'
    elif typeNum == 5:
        return 'ui16'
    elif typeNum == 6:
        return 'ui32'
    elif typeNum == 7:
        return 'ui64'
    elif typeNum == 8:
        return 'f32'
    elif typeNum == 9:
        return 'f64'
    else:
        return False

def getTypeSize(typeNum):
    if typeNum == 0: # int8_t
        return 1
    elif typeNum == 1: # int16_t
        return 2
    elif typeNum == 2: # int32_t
        return 4
    elif typeNum == 3: # int64_t
        return 8
    elif typeNum == 4: # uint8_t
        return 1
    elif typeNum == 5: # uint16_t
        return 2
    elif typeNum == 6: # uint32_t
        return 4
    elif typeNum == 7: # uint64_t
        return 8
    elif typeNum == 8: # f32
        return 4
    elif typeNum == 9: # f64
        return 8
    else:
        return False

def getTypeNum(typeString):
    if typeString == 'i8': # int8_t
        return 0
    elif typeString == 'i16': # int16_t
        return 1
    elif typeString == 'i32': # int32_t
        return 2
    elif typeString == 'i64': # int64_t
        return 3
    elif typeString == 'ui8': # uint8_t
        return 4
    elif typeString == 'ui16': # uint16_t
        return 5
    elif typeString == 'ui32': # uint32_t
        return 6
    elif typeString == 'ui64': # uint64_t
        return 7
    elif typeString == 'f32': # f32
        return 8
    elif typeString == 'f64': # f64
        return 9
    elif typeString == 's': # String
        return 15
    elif typeString == 'int8': # int8_t
        return 0
    elif typeString == 'int16': # int16_t
        return 1
    elif typeString == 'int32': # int32_t
        return 2
    elif typeString == 'int64': # int64_t
        return 3
    elif typeString == 'uint8': # uint8_t
        return 4
    elif typeString == 'uint16': # uint16_t
        return 5
    elif typeString == 'uint32': # uint32_t
        return 6
    elif typeString == 'uint64': # uint64_t
        return 7
    elif typeString == 'float32': # f32
        return 8
    elif typeString == 'float64': # f64
        return 9
    elif typeString == 's': # String
        return 15
    else:
        return None

def getStTypeList(formatString):
    typeList = list()
    for typeStr in formatString.split(','):
        if typeStr=='':
            return None

        s = typeStr.split('x')
        if len(s) is not 2:
            return None
        elif s[0]=='' or s[1]=='':
            return None

        type = getTypeNum(s[0])
        if type is None:
            return None

        try:
            num = int(s[1])
        except ValueError as e:
            return None

        typeList.append({'type':type, 'num':num})
    return typeList

def getStDtype(formatString):
    args = list()
    for i, typeStr in enumerate(formatString.split(',')):
        if typeStr=='':
            return None

        s = typeStr.split('x')
        if len(s) is not 2:
            return None
        elif s[0]=='' or s[1]=='':
            return None

        type = getTypeNum(s[0])
        if type is None:
            return None

        try:
            num = int(s[1])
        except ValueError as e:
            return None

        args.append(('f'+str(i),getNpType(type),(num,)))
    dt = np.dtype(args)
    return dt


def getStSize(typeList):
    size = 0
    for t in typeList:
        size = size + getTypeSize(t['type']) * t['num']
    return size

def decode_struct(formatString, data):
    dt = getStDtype(formatString)
    if dt is None:
        return None
    elif dt.itemsize != len(data):
        return None
    res = np.frombuffer(data, dtype=dt, count=1)
    return np.asarray(res[0], dtype=dt)

def decode_array(typeNum, data):
    return np.frombuffer(data, dtype=getNpType(typeNum))

def npDtypeToFs(dt):
    res  = ''
    last = len(dt) - 1
    for i in range(len(dt)):
        type = getTypeStr(getTypeNum(dt[i].base.name))
        num  = dt[i].shape[0]
        res += type + 'x' + str(num)
        if i != last:
            res += ','
    return res

def getFs(dt):
    return npDtypeToFs(dt)



# -*- coding: utf-8 -*-

from . import general
from . import packet
import enum

__all__  = ['Decoder']

class _Status(enum.IntEnum):
    header = 0
    addr   = 1
    seq    = 2
    func   = 3
    byte   = 4
    data   = 5
    chksum = 6

class Decoder():

    class _State():
        status = _Status(_Status.header)
        addr   = int(0)
        seq    = int(0)
        func   = int(0)
        dbytes = int(0)
        data   = bytes()

        chksum = int(0)
        count = int(0)

    _text    = bytes([])
    _usedNum = int(0)

    def __init__(self):
        pass

    def putText(self, text):
        self._text += text

    def _get_ch(self):
        if self._usedNum+1 <= len(self._text):
            self._usedNum = self._usedNum + 1
            return self._text[self._usedNum-1]
        else:
            raise IndexError('get ch out of range')

    def de(self):
        for i in range(len(self._text)-self._usedNum):
            res = self._step()
            if res:
                return True
        return False

    def _step(self):
        ch = self._get_ch()
        if self._State.status == _Status.header:
            self._State.chksum = 0
            self._State.count  = 0
            self._State.status = _Status.addr

        elif self._State.status == _Status.addr:
            self._State.chksum += ch
            self._State.count  += 1
            if self._State.count == 1:
                self._State.addr = ch
            else:
                self._State.addr = (self._State.addr<<8) + ch
                self._State.count = 0
                self._State.status = _Status.seq

        elif self._State.status == _Status.seq:
            self._State.chksum += ch
            self._State.seq    = ch
            self._State.status  = _Status.func

        elif self._State.status == _Status.func:
            self._State.chksum += ch
            self._State.func   = ch
            self._State.status = _Status.byte

        elif self._State.status == _Status.byte:
            self._State.chksum += ch
            self._State.count  += 1
            if self._State.count == 1:
                self._State.dbytes = ch
            else:
                self._State.dbytes = (self._State.dbytes<<8) + ch
                self._State.count = 0

                if self._State.dbytes != 0:
                    self._State.data   = bytes([])
                    self._State.status = _Status.data
                else:
                    self._State.status = _Status.chksum

        elif self._State.status == _Status.data:
            self._State.chksum += ch
            self._State.count  += 1
            self._State.data   += bytes([ch])
            if self._State.count == self._State.dbytes:
                self._State.count = 0
                self._State.status = _Status.chksum

        elif self._State.status == _Status.chksum:
            if self._State.chksum&0xFF == ch:
                self._State.status = _Status.header
                return True
            else:
                self._State.status = _Status.header
        return False

    def getPac(self):
        en = packet.Packet()
        en.setAddr(self._State.addr)
        en.setSeq (self._State.seq )
        en.setFunc(self._State.func)
        en.setData(self._State.data)
        return en



from .type import *
__all__  = ['Decoder']

#  header of hmi get array
_CONST_HEADER_GET_AR = b'\xaa\xaa\xaa'

#  header of hmi get array
_CONST_HEADER_GET_ST = b'\xbb\xbb\xbb'


class Decoder(object):
    class State(object):
        status    = int(0)
        dataBytes = int(0)
        sBytes    = int(0)
        chksum    = int(0)
        count     = int(0)
        header    = bytes(3)
        databuf   = bytes()
        databuf2  = bytes()

    _text = bytes()
    _idx  = int(0)
    _state = State()
    _en_utf8 = bool(True)
    _en_big5 = bool(True)

    """docstring for Decoder."""
    def __init__(self):
        super(Decoder, self).__init__()
        self._idx = int(0)

    def add_text(self, text):
        self._text += text

    def set_text(self, text):
        self._text = text

    def get_text(self):
        return self._text

    def _step(self):
        if len(self._text) == 0:
            return 0, None
        for ch in self._text[self._idx::]:
            self._idx += 1
            if self._state.status == 0:
                # print('state is ' + str(self._state.status))
                if bytes([ch]) == b'\r' or bytes([ch]) == b'\n':
                    if len(self._state.databuf) > 0:
                        res = self._decodeDatabufToStr()
                        return res
                else:
                    self._state.databuf += bytes([ch])
                    self._state.header = self._state.header[1:3] + bytes([ch])
                #
                if   self._state.header == _CONST_HEADER_GET_AR:
                    # remove 3 bytes header from databuf
                    self._state.databuf = self._state.databuf[0:-3]
                    if len(self._state.databuf) > 0:
                        res = self._decodeDatabufToStr()
                        self._resetState()
                        self._state.status = 10
                        return res
                    self._resetState()
                    self._state.status = 10
                elif self._state.header == _CONST_HEADER_GET_ST:
                    # remove 3 bytes header from databuf
                    self._state.databuf = self._state.databuf[0:-3]
                    if len(self._state.databuf) > 0:
                        res = self._decodeDatabufToStr()
                        self._resetState()
                        self._state.status = 10
                        return res
                    self._resetState()
                    self._state.status = 20
            elif self._state.status == 10:
                # print('state is ' + str(self._state.status))
                # arrTypeNum
                if ch > 9:
                    print('arrTypeNum error')
                    raise
                self._state.arrTypeNum = ch
                self._state.header   = bytes(3)
                self._state.status   = 11
                self._state.dataBytes = 0
                self._state.count    = 0
                self._state.chksum   = 0
            elif self._state.status == 11:
                # print('state is ' + str(self._state.status))
                self._state.dataBytes = (self._state.dataBytes<<8) + ch
                self._state.chksum  += ch
                self._state.count += 1
                if self._state.count == 2:
                    self._state.status  = 12
                    self._state.count   = 0
                    self._state.databuf = bytes()
            elif self._state.status == 12:
                # print('state is ' + str(self._state.status))
                self._state.chksum  += ch
                self._state.databuf += bytes([ch])
                self._state.count   += 1
                if self._state.count == self._state.dataBytes:
                    self._state.status  = 13
                    self.databuf = decode_array(self._state.arrTypeNum,self._state.databuf)
            elif self._state.status == 13:
                # print('state is ' + str(self._state.status))
                if self._state.chksum&0xFF == ch:
                    self._state.databuf = bytes()
                    self._state.status = 0
                    return 1, self.databuf
                else:
                    print('arrChkSum error')
                    raise

            elif self._state.status == 20:
                # print('state is ' + str(self._state.status))
                # total bytes
                self._state.dataBytes = (self._state.dataBytes<<8) + ch
                self._state.chksum  += ch
                self._state.count   += 1
                if self._state.count == 2:
                    self._state.status  = 21
                    self._state.databuf = bytes()

            elif self._state.status == 21:
                # print('state is ' + str(self._state.status))
                # bytes of format string
                self._state.chksum  += ch
                self._state.sBytes   = ch
                self._state.status   = 22
                self._state.count    = 0
                self._state.dataBytes -= ch+1
            elif self._state.status == 22:
                # print('state is ' + str(self._state.status))
                # formatString
                self._state.chksum   += ch
                self._state.databuf2 += bytes([ch])
                self._state.count    += 1
                if self._state.count is self._state.sBytes:
                    self._state.status = 23
                    self._state.count  = 0
                    self._state.formatString = self._state.databuf2.decode("ascii")
                    self._state.databuf = bytes()

            elif self._state.status == 23:
                # print('state is ' + str(self._state.status))
                # data
                self._state.chksum  += ch
                self._state.databuf += bytes([ch])
                self._state.count   += 1
                if self._state.count == self._state.dataBytes:
                    self._state.status  = 24

            elif self._state.status == 24:
                # print('state is ' + str(self._state.status))
                # chksum
                if self._state.chksum&0xFF == ch:
                    self._state.status = 0
                    data = decode_struct( self._state.formatString, self._state.databuf )
                    self._state.databuf = bytes()
                    return 2, data
                else:
                    print('stChkSum error')
                    raise
        return 0, None

    def _decodeDatabufToStr(self):
        string = None
        if string is None and self._en_utf8:
            string = _trans_line_to_utf8(self._state.databuf)
        if string is None and self._en_big5:
            string = _trans_line_to_big5(self._state.databuf)
        if string is None:
            string = _trans_line_to_ascii(self._state.databuf)
        self._state.databuf = bytes()
        if string is None:
            return 0, None
        else:
            return 3, string

    def _resetState(self):
        self._state = self.State()

    def get(self):
        type, data = self._step()
        if type is -1:
            # TODO handle ERROR here
            pass
        elif type is 0 :
            return 0, None
        else:
            self.set_text(self._text[self._idx::])
            self._idx = 0
            return type, data

def _trans_line_to_utf8(line):
    try:
        return line.decode('utf8')
    except Exception as e:
        return None

def _trans_line_to_big5(line):
    try:
        return line.decode('big5')
    except Exception as e:
        return None

def _trans_line_to_ascii(line):
    try:
        return line.decode('ascii')
    except Exception as e:
        return None



from .type import *
import numpy as np

__all__ = ['encodeArToPac', 'encodeStToPac']

_HEADER = b'\xab\xab\xab'

def encodeArToPac(data):
    pac  = _HEADER
    type = getTypeNum(data.dtype.name)
    data = data.tobytes()
    l    = len(data)
    pac += bytes([type, l>>8, l&0xFF])
    pac += data
    pac += bytes([sum(data)&0xFF])
    return pac

def encodeStToPac(data):
    pac  = _HEADER
    fs   = bytes(getFs(data.dtype), encoding='ascii')
    data = data.tobytes()
    l    = len(data) + len(fs) + 1
    pac += bytes([l>>8, l&0xFF, len(fs)])
    pac += fs
    pac += data
    pac += bytes([sum(data)&0xFF])
    return pac



from setuptools import setup, find_packages

REQUIREMENTS = [
    'setuptools',
    'pyserial',
    'numpy',
]

setup(
    name='asa-hmi-api',
    version='0.1.1',
    description = '',
    long_description='',
    author = 'mickey9910326',
    author_email = 'mickey9910326@gmail.com',
    url='https://gitlab.com/mickey9910326/hmi-api-py',
    license = 'MIT',
    packages=find_packages(),
    zip_safe=False,
    entry_points = {},
    install_requires=REQUIREMENTS
)



m# -*- coding: utf-8 -*-

from . import general

__all__  = ['Packet']

class Packet():
    _seq = int(0)
    _isContinued = False
    _addr = int(0)
    _func = int(0)
    _data = bytes([])

    def __init__(self):
        pass

    def toByte(self):
        seq     = self._seq + (int(self._isContinued)<<5)
        dataLen = len(self._data)
        b  = general.HEADER
        b += bytes([self._addr>>8, self._addr&0xFF])
        b += bytes([seq])
        b += bytes([self._func])
        b += bytes([dataLen>>8, dataLen&0xFF])
        b += self._data
        chksum = sum(b) - int(general.HEADER[0])
        b += bytes([chksum])
        return b

    def setAddr(self, addr):
        if addr<65535 :
            self._addr = addr
        else:
            raise ValueError(str(addr) + ' excceds max value of address (65535).')

    def setSeq(self, seq):
        if seq<31 :
            self._seq = seq
            self._isContinued = True
        else:
            raise ValueError(str(seq) + ' excceds max num of seq (31).')

    def setNonContinue(self):
        self._seq = 0
        self._isContinued = False

    def setFunc(self, func):
        self._func = func

    def setData(self, data):
        self._data = data

    def setAsAck(self):
        self._func = 2
        self.setNonContinue()
        self._data = bytes([])

    def setAsNack(self):
        self._func = 3
        self.setNonContinue()
        self._data = bytes([])
